function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

var clone = function clone(data) {
  if (typeof data !== 'object') throw new Error('lottie data type error!');
  return JSON.parse(JSON.stringify(data));
};
var merge = function merge(left, right) {
  return _extends({}, left, right);
};
var defaultDropKeyList = ['nm', 'tyName', 'n', 'mn', 'cl', 'ln'];
var defaultConfig = {
  copy: false,
  dropKeyList: defaultDropKeyList,
  numberFixLength: 3
};
var isNil = function isNil(x) {
  return x === null || x === undefined;
};
var fixed = function fixed(length) {
  var r = Math.pow(10, length);
  return function (x) {
    return Math.round(x * r) / r;
  };
};
var isNumber = function isNumber(n) {
  return typeof n === 'number';
};

var minify = function minify(data, config) {
  if (config === void 0) {
    config = {};
  }

  config = merge(defaultConfig, config);
  var _config = config,
      copy = _config.copy;

  if (copy) {
    data = clone(data);
  }

  walk(data, config);
  fixAttrIndIsUndefined(data);
  return data;
};

var walk = function walk(_data, config) {
  var _config$dropKeyList = config.dropKeyList,
      dropKeyList = _config$dropKeyList === void 0 ? [] : _config$dropKeyList,
      _config$numberFixLeng = config.numberFixLength,
      numberFixLength = _config$numberFixLeng === void 0 ? 3 : _config$numberFixLeng;
  var numberFixFun = fixed(numberFixLength);
  var refIdList = [];

  var getRefId = function getRefId(id) {
    var i = refIdList.indexOf(id);
    if (i !== -1) return i.toString(36);
    refIdList.push(id);
    return (refIdList.length - 1).toString(36);
  };

  var dfs = function dfs(data) {
    if (isNil(data)) throw new Error('lottie value is null or undefined');
    if (typeof data === 'string') return;
    if (typeof data === 'number') return;

    if (Array.isArray(data)) {
      data.forEach(function (item, k) {
        if (isNumber(item)) {
          data[k] = numberFixFun(item);
        } else {
          dfs(item);
        }
      });
    } else if (typeof data === 'object') {
      for (var _iterator = _createForOfIteratorHelperLoose(dropKeyList), _step; !(_step = _iterator()).done;) {
        var _k = _step.value;

        if (_k in data && data[_k]) {
          delete data[_k];
        }
      }

      for (var k in data) {
        if (isNumber(data[k])) {
          data[k] = numberFixFun(data[k]);
        } else if (k === 'refId') {
          data['refId'] = getRefId(data[k]);
        } else if (k === 'id') {
          data['id'] = getRefId(data[k]);
        } else {
          dfs(data[k]);
        }
      }
    }
  };

  dfs(_data);
}; // from https://github1s.com/fancy-lottie/lottie-compress/blob/HEAD/src/main.ts


var fixAttrIndIsUndefined = function fixAttrIndIsUndefined(data) {
  data.layers.forEach(function (layer, index) {
    if (layer.ind === undefined) {
      layer.ind = index + 1;
    }
  });
  data.assets.forEach(function (asset) {
    if (asset.layers) {
      asset.layers.forEach(function (layer, index) {
        if (layer.ind === undefined) {
          layer.ind = index + 1;
        }
      });
    }
  });
  return data;
};

export { minify };
//# sourceMappingURL=lottie-minify.esm.js.map
